from colab.widgets.widget_manager import WidgetManager
from colab_discourse.widgets.home_section import DiscourseHomeSectionWidget
from colab_discourse.widgets.navigation_links import (
    DiscourseNavigationLinksWidget
)


WidgetManager.register_widget('home_section', DiscourseHomeSectionWidget())
WidgetManager.register_widget('navigation_links',
                              DiscourseNavigationLinksWidget())



name = 'colab_discourse'
verbose_name = 'Colab Discourse Plugin Plugin'

upstream = 'http://127.0.0.1:8080/expressao/'
api_key = 'api_key'
api_username = 'system'
sso_secret = 'sso_secret'

urls = {
    'include': 'colab_discourse.urls',
    'prefix': '^expressao/',
    'login': '/expressao/accounts/login/',
}

settings_variables = {
    'COLAB_STATICS': [
        '/colab-plugins/discourse/src/colab_discourse/static'
    ]
}



from django.contrib.auth.signals import user_logged_in, user_logged_out
from colab.plugins.utils.apps import ColabPluginAppConfig
from colab_discourse.tasks import login_user, logout_user


class DiscourseAppConfig(ColabPluginAppConfig):
    name = 'colab_discourse'
    verbose_name = 'Colab Discourse Plugin'
    short_name = 'discourse'
    namespace = 'discourse'

    def connect_signal(self):
        user_logged_in.connect(login_user)
        user_logged_out.connect(logout_user)



from haystack import indexes
from colab_discourse import models


class DiscourseTopicIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.EdgeNgramField(document=True, use_template=True,
                                  stored=False)
    type = indexes.EdgeNgramField()

    # Model fields
    title = indexes.EdgeNgramField(model_attr='title')
    category = indexes.EdgeNgramField(model_attr='category__name')
    category_color = indexes.CharField(model_attr='category__color')
    category_text_color = indexes.CharField(model_attr='category__text_color')
    created_at = indexes.DateTimeField(model_attr='created_at')
    created_by = indexes.EdgeNgramField(model_attr='created_by__username')
    last_posted_at = indexes.DateTimeField(model_attr='last_posted_at')
    views = indexes.IntegerField(model_attr='views')
    like_count = indexes.IntegerField(model_attr='like_count')
    posts_count = indexes.IntegerField(model_attr='posts_count')
    participant_count = indexes.IntegerField(model_attr='participant_count')
    visible = indexes.BooleanField(model_attr='visible')
    closed = indexes.BooleanField(model_attr='closed')
    url = indexes.EdgeNgramField(model_attr='get_url')

    def get_model(self):
        return models.DiscourseTopic

    def prepare_type(self, obj):
        return u'topic'

    def index_queryset(self, using=None):
        return self.get_model().objects.filter(visible=True)


class DiscoursePostIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.EdgeNgramField(document=True, use_template=True,
                                  stored=False)
    type = indexes.EdgeNgramField()

    # Model fields
    author = indexes.EdgeNgramField(model_attr='author__username')
    topic = indexes.EdgeNgramField(model_attr='topic__title')
    cooked = indexes.EdgeNgramField(model_attr='cooked')
    created_at = indexes.DateTimeField(model_attr='created_at')
    updated_at = indexes.DateTimeField(model_attr='updated_at')
    reply_count = indexes.IntegerField(model_attr='reply_count')
    quote_count = indexes.IntegerField(model_attr='quote_count')
    reads = indexes.IntegerField(model_attr='reads')
    url = indexes.EdgeNgramField(model_attr='get_url')

    def get_model(self):
        return models.DiscoursePost

    def prepare_type(self, obj):
        return u'post'

    def index_queryset(self, using=None):
        return self.get_model().objects.filter(topic__visible=True)




from django.conf.urls import patterns, url

import views

urlpatterns = patterns(
    '',
    url(r'^(?P<path>.*)$', views.ColabDiscoursePluginProxyView.as_view(),
        name='colab_discourse'),
)



from colab.widgets.widget_manager import Widget
from django.template.loader import render_to_string
from colab_discourse.models import DiscourseTopic


class DiscourseHomeSectionWidget(Widget):
    name = 'Discourse Home Section'
    template = 'widgets/discourse_home_section.html'

    def generate_content(self, **kwargs):
        context = kwargs.get('context')
        discourse_data = DiscourseTopic.objects.filter(visible=True)
        discourse_data = discourse_data.order_by('-last_posted_at')
        context['discourse_data'] = discourse_data[:10]

        self.content = render_to_string(self.template, context)
        return kwargs.get('context')



from django.contrib.sites.models import Site
from django.db.models.signals import post_save
from django.dispatch import receiver
from colab.plugins import helpers
from colab.accounts.models import User
from pydiscourse.sso import sso_validate, sso_redirect_url
import requests
import re


def perform_discourse_login(user):
    plugin_config = helpers.get_plugin_config('colab_discourse')
    prefix = helpers.get_plugin_prefix('colab_discourse', regex=False)
    base_url = Site.objects.get_current().domain
    base_url = "{}/{}".format(base_url, prefix)
    url = base_url + "/session/sso"

    response = requests.get(url, allow_redirects=False)
    if response.status_code == 302:
        location = response.headers.get("Location")

        regex = re.compile(r"sso=(.+)&sig=(.+)")
        matches = regex.search(location)
        payload, signature = matches.groups()
        secret = plugin_config.get('sso_secret')

        nonce = sso_validate(payload, signature, secret)
        url = sso_redirect_url(nonce, secret, user.email,
                               user.id, user.username,
                               name=user.first_name.encode('utf-8'))
        response = requests.get(base_url + url, allow_redirects=False)
        return response
    return None


def login_user(sender, user, request, **kwargs):
    response = perform_discourse_login(user)
    if response:
        session = response.cookies.get('_forum_session')
        t_cookie = response.cookies.get('_t')
        request.COOKIES.set("_forum_session", session)
        request.COOKIES.set("_t", t_cookie)


def logout_user(sender, user, request, **kwargs):
    request.COOKIES.delete('_forum_session')
    request.COOKIES.delete('_t')


@receiver(post_save, sender=User)
def create_discourse_accounts(sender, instance, created, **kwargs):
    if instance.is_active and created:
        perform_discourse_login(instance)



from colab_discourse import models
from colab.plugins.data import PluginDataImporter
from colab.plugins import helpers
from django.db.models.fields import DateTimeField
from django.utils.dateparse import parse_datetime
from django.contrib.auth import get_user_model
from pydiscourse.client import DiscourseClient
User = get_user_model()


class ColabDiscoursePluginDataImporter(PluginDataImporter):
    app_label = 'colab_discourse'

    def __init__(self):
        plugin_config = helpers.get_plugin_config('colab_discourse')
        self.client = DiscourseClient(
            plugin_config['upstream'],
            api_key=plugin_config['api_key'],
            api_username=plugin_config['api_username']
        )

    def fill_object_data(self, model_class, data):
        try:
            obj = model_class.objects.get(id=data['id'])
        except model_class.DoesNotExist:
            obj = model_class()

        for field in obj._meta.fields:
            try:
                if field.name == 'badge_type':
                    obj.badge_type_id = data['badge_type_id']
                    continue

                if field.name == 'category':
                    obj.category_id = data['category_id']
                    continue

                if field.name == 'author':
                    user = User.objects.get(username=data['username'])
                    obj.author = user
                    continue

                if field.name == 'topic':
                    obj.topic_id = data['topic_id']
                    continue

                if isinstance(field, DateTimeField):
                    value = parse_datetime(data[field.name])
                else:
                    value = data[field.name]

                setattr(obj, field.name, value)
            except KeyError:
                continue

        return obj

    def fetch_data(self):
        models.DiscourseCategory.objects.all().delete()
        models.DiscourseTopic.objects.all().delete()
        models.DiscoursePost.objects.all().delete()
        models.DiscourseBadgeType.objects.all().delete()
        models.DiscourseBadge.objects.all().delete()

        self.fetch_categories()
        self.fetch_badges()
        self.fetch_topics()
        self.fetch_posts()

    def fetch_topics(self):
        topics_data = self.get_all_topics()
        for topic_data in topics_data:
            topic = self.fill_object_data(models.DiscourseTopic, topic_data)
            topic = self.complete_topic_informations(topic)

    def complete_topic_informations(self, topic):
        topic_data = self.client.topic(topic.slug, topic.id)
        created_by = topic_data['details']['created_by']['username']
        topic.created_by = User.objects.get(username=created_by)
        topic.participant_count = topic_data['participant_count']
        topic.save()

        for user_data in topic_data['details']['participants']:
            participant = User.objects.get(username=user_data['username'])
            topic.participants.add(participant)

        return topic

    def fetch_posts(self):
        topics = models.DiscourseTopic.objects.all()
        for topic in topics:
            self.fetch_posts_by_topic(topic)

    def fetch_posts_by_topic(self, topic):
        topic_info = self.client.posts(topic.id)
        posts_data = topic_info['post_stream']['posts']
        for post_data in posts_data:
            post = self.fill_object_data(models.DiscoursePost, post_data)
            post.save()

    def get_all_topics(self, page=0):
        topics = self.client.latest_topics(page=page)
        topics = topics['topic_list']['topics']
        all_topics = []
        all_topics.extend(topics)

        if topics:
            page += 1
            all_topics.extend(self.get_all_topics(page=page))

        return all_topics

    def fetch_categories(self):
        categories = self.client.categories()
        for category_data in categories:
            if category_data['slug'] != 'uncategorized':
                category = self.fill_object_data(models.DiscourseCategory,
                                                 category_data)
                category.save()

    def fetch_badges(self):
        badges_data = self.client.badges()
        self.fetch_badge_types(badges_data['badge_types'])
        for data in badges_data['badges']:
            badge = self.fill_object_data(models.DiscourseBadge, data)
            badge.save()

    def fetch_badge_types(self, data):
        for badge_type_data in data:
            badge_type = self.fill_object_data(models.DiscourseBadgeType,
                                               badge_type_data)
            badge_type.save()



# Your models here.
from django.db import models
from django.conf import settings
from colab.plugins import helpers


class DiscourseCategory(models.Model):

    id = models.IntegerField(primary_key=True)
    name = models.CharField(max_length=255)
    color = models.CharField(max_length=50)
    text_color = models.CharField(max_length=50)
    slug = models.CharField(max_length=255)
    description = models.TextField(null=True)

    topic_count = models.IntegerField(default=0)
    post_count = models.IntegerField(default=0)

    class Meta:
        ordering = ['name']


class DiscourseTopic(models.Model):

    id = models.IntegerField(primary_key=True)
    title = models.CharField(max_length=255)
    slug = models.CharField(max_length=255)
    category = models.ForeignKey('DiscourseCategory')

    created_at = models.DateTimeField()
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL)
    last_poster_username = models.CharField(max_length=255)
    last_posted_at = models.DateTimeField()
    participants = models.ManyToManyField(settings.AUTH_USER_MODEL,
                                          related_name='topics')

    views = models.IntegerField(default=0)
    like_count = models.IntegerField(default=0)
    posts_count = models.IntegerField(default=0)
    participant_count = models.IntegerField(default=0)

    visible = models.BooleanField(default=True)
    closed = models.BooleanField(default=False)

    def get_url(self):
        prefix = helpers.get_plugin_prefix('colab_discourse', regex=False)
        return '/{}t/{}/{}'.format(prefix, self.slug, self.id)

    def get_category_url(self):
        prefix = helpers.get_plugin_prefix('colab_discourse', regex=False)
        return '/{}c/{}'.format(prefix, self.category.slug)


class DiscoursePost(models.Model):
    id = models.IntegerField(primary_key=True)
    author = models.ForeignKey(settings.AUTH_USER_MODEL)
    topic = models.ForeignKey('DiscourseTopic', related_name='posts')
    post_number = models.IntegerField()
    created_at = models.DateTimeField()
    updated_at = models.DateTimeField()
    cooked = models.TextField()

    reply_count = models.IntegerField(default=0)
    quote_count = models.IntegerField(default=0)
    reads = models.IntegerField(default=0)

    def get_url(self):
        prefix = helpers.get_plugin_prefix('colab_discourse', regex=False)
        return '/{}t/{}/{}#post_{}'.format(prefix, self.topic.slug,
                                           self.topic_id, self.id)


class DiscourseBadgeType(models.Model):

    id = models.IntegerField(primary_key=True)
    name = models.CharField(max_length=255)


class DiscourseBadge(models.Model):

    id = models.IntegerField(primary_key=True)
    badge_type = models.ForeignKey('DiscourseBadgeType')
    name = models.CharField(max_length=255)
    slug = models.CharField(max_length=255)
    description = models.CharField(max_length=255)
    long_description = models.TextField()
    icon = models.CharField(max_length=255)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DiscourseBadge',
            fields=[
                ('id', models.IntegerField(serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=255)),
                ('slug', models.CharField(max_length=255)),
                ('description', models.CharField(max_length=255)),
                ('long_description', models.TextField()),
                ('icon', models.CharField(max_length=255)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DiscourseBadgeType',
            fields=[
                ('id', models.IntegerField(serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=255)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DiscourseCategory',
            fields=[
                ('id', models.IntegerField(serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=255)),
                ('color', models.CharField(max_length=50)),
                ('text_color', models.CharField(max_length=50)),
                ('slug', models.CharField(max_length=255)),
                ('description', models.CharField(max_length=255, null=True)),
                ('topic_count', models.IntegerField(default=0)),
                ('post_count', models.IntegerField(default=0)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DiscoursePost',
            fields=[
                ('id', models.IntegerField(serialize=False, primary_key=True)),
                ('post_number', models.IntegerField()),
                ('created_at', models.DateTimeField()),
                ('updated_at', models.DateTimeField()),
                ('cooked', models.TextField()),
                ('reply_count', models.IntegerField(default=0)),
                ('quote_count', models.IntegerField(default=0)),
                ('reads', models.IntegerField(default=0)),
                ('author', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DiscourseTopic',
            fields=[
                ('id', models.IntegerField(serialize=False, primary_key=True)),
                ('title', models.CharField(max_length=255)),
                ('slug', models.CharField(max_length=255)),
                ('created_at', models.DateTimeField()),
                ('last_poster_username', models.CharField(max_length=255)),
                ('last_posted_at', models.DateTimeField()),
                ('views', models.IntegerField(default=0)),
                ('like_count', models.IntegerField(default=0)),
                ('posts_count', models.IntegerField(default=0)),
                ('participant_count', models.IntegerField(default=0)),
                ('visible', models.BooleanField(default=True)),
                ('closed', models.BooleanField(default=False)),
                ('category', models.ForeignKey(to='colab_discourse.DiscourseCategory')),
                ('created_by', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
                ('participants', models.ManyToManyField(related_name='topics', to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='discoursepost',
            name='topic',
            field=models.ForeignKey(related_name='posts', to='colab_discourse.DiscourseTopic'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='discoursebadge',
            name='badge_type',
            field=models.ForeignKey(to='colab_discourse.DiscourseBadgeType'),
            preserve_default=True,
        ),
    ]



from django.core.management.base import BaseCommand
from colab_discourse.data_importer import ColabDiscoursePluginDataImporter


class Command(BaseCommand):
    def handle(self, *args, **options):
        discourse_data = ColabDiscoursePluginDataImporter()
        discourse_data.fetch_data()
